package nl.kallestruik.dtweaks.fakeplayer

import com.mojang.authlib.GameProfile
import net.minecraft.network.chat.Component
import net.minecraft.network.protocol.PacketFlow
import net.minecraft.network.protocol.game.ClientboundRotateHeadPacket
import net.minecraft.network.protocol.game.ClientboundTeleportEntityPacket
import net.minecraft.server.MinecraftServer
import net.minecraft.server.TickTask
import net.minecraft.server.level.ServerLevel
import net.minecraft.server.level.ServerPlayer
import net.minecraft.world.damagesource.DamageSource
import net.minecraft.world.food.FoodData
import net.minecraft.world.level.block.entity.SkullBlockEntity
import nl.kallestruik.dtweaks.DTweaks
import org.bukkit.Bukkit
import org.bukkit.Location
import org.bukkit.craftbukkit.v1_20_R1.CraftWorld
import java.util.concurrent.atomic.AtomicReference

class FakePlayer(
    server: MinecraftServer,
    private val serverLevel: ServerLevel,
    gameProfile: GameProfile
): ServerPlayer(server, serverLevel, gameProfile) {
    val locale = "en_US"

    companion object {
        fun atLocation(location: Location, name: String): FakePlayer? {
            // Get the minecraft server instance.
            val minecraftServer = MinecraftServer.getServer()
            // Get the world
            val serverLevel = (location.world as CraftWorld).handle
            // Get the game profile from the cache (or retrieve it if it is not cached)
            var gameProfile = minecraftServer.profileCache?.get(name).takeIf { it?.isPresent == true }?.get() ?: return null
            if (gameProfile.properties.containsKey("textures")) {
                val result = AtomicReference<GameProfile>()
                SkullBlockEntity.updateGameprofile(gameProfile, result::set)
                gameProfile = result.get()
            }

            // Create an instance of our fake player.
            val instance = FakePlayer(minecraftServer, serverLevel, gameProfile)

            // Create a fake connection for our fake player.
            val connection = FakeConnection(PacketFlow.SERVERBOUND)

            val bukkitPlayer = instance.bukkitEntity

            Bukkit.getScheduler().runTaskLater(DTweaks.plugin, Runnable {
                bukkitPlayer.spigot().respawn()
                bukkitPlayer.teleport(location)
            }, 0)

            minecraftServer.playerList.placeNewPlayer(connection, instance)

            minecraftServer.playerList.broadcastAll(
                ClientboundRotateHeadPacket(
                    instance,
                    (instance.yHeadRot * 256 / 360).toInt().toByte()
                ), serverLevel.dimension())

            minecraftServer.playerList.broadcastAll(ClientboundTeleportEntityPacket(instance), serverLevel.dimension())

            instance.entityData.set(DATA_PLAYER_MODE_CUSTOMISATION, 0x7f)
            return instance
        }
    }

    override fun kill() {
        server.tell(TickTask(server.tickCount) {
            connection.onDisconnect(Component.literal("Killed"))
        })
    }

    override fun die(source: DamageSource) {
        super.die(source)
        heal(maxHealth)
        foodData = FoodData(this)
        kill()
    }

    override fun tick() {
        super.tick()

        if (server.tickCount % 10 == 0) {
            connection.resetPosition()
            serverLevel.chunkSource.move(this)
            hasChangedDimension()
        }
    }

//    override fun die(cause: DamageSource?) {
//        super.die(cause)
//        health = 20f
//        foodData = FoodMetaData(this)
//        killEntity()
//    }

    override fun getIpAddress() = "127.0.0.1"
}