package nl.kallestruik.dtweaks.managers

import org.bukkit.NamespacedKey
import org.bukkit.configuration.InvalidConfigurationException
import org.bukkit.configuration.file.YamlConfiguration
import java.io.File
import java.io.IOException

class NoSpawnZoneManager {
    lateinit var file: File
    var noSpawnZones: MutableMap<NamespacedKey, MutableSet<Pair<Int, Int>>> = mutableMapOf()

    @Throws(IOException::class, InvalidConfigurationException::class)
    fun loadFromFile(file: File) {
        this.file = file
        file.parentFile.mkdirs()
        file.createNewFile()
        val config = YamlConfiguration()
        config.load(file)
        noSpawnZones.clear()

        for (world in config.getKeys(false)) {
            val worldKey = NamespacedKey.fromString(world)!!
            noSpawnZones[worldKey] = mutableSetOf()
            for (item in config.getStringList(world)) {
                val parts = item.split(":")
                if (parts.size != 2)
                    continue

                noSpawnZones[worldKey]?.add(Pair(parts[0].toInt(), parts[1].toInt()))
            }
        }
    }

    @Throws(IOException::class, InvalidConfigurationException::class)
    fun saveToFile() {
        file.parentFile.mkdirs()
        file.createNewFile()
        val config = YamlConfiguration()
        config.load(file)

        for (entry in noSpawnZones) {
            val world = entry.key.toString()
            val list = mutableListOf<String>()
            for (chunk in entry.value) {
                list.add("${chunk.first}:${chunk.second}")
            }

            config[world] = list
        }

        config.save(file)
    }

    fun isChunkClaimed(x: Int, z: Int, world: NamespacedKey): Boolean {
        return noSpawnZones[world]?.contains(Pair(x, z)) ?: false
    }

    fun claimChunk(x: Int, z: Int, world: NamespacedKey) {
        val chunks = noSpawnZones.getOrDefault(world, mutableSetOf())
        chunks.add(Pair(x, z))
        noSpawnZones[world] = chunks
    }

    fun unclaimChunk(x: Int, z: Int, world: NamespacedKey) {
        val chunks = noSpawnZones.getOrDefault(world, mutableSetOf())
        chunks.remove(Pair(x, z))
        noSpawnZones[world] = chunks
    }
}