package nl.kallestruik.dtweaks.tweaks.craftingtweaks

import nl.kallestruik.dtweaks.tweaks.ITweak
import org.bukkit.Bukkit
import org.bukkit.Material
import org.bukkit.NamespacedKey
import org.bukkit.inventory.ItemStack
import org.bukkit.inventory.ShapedRecipe
import org.bukkit.plugin.java.JavaPlugin

class CraftableChainmail(
    private val plugin: JavaPlugin
): ITweak {
    private var helmetRecipeKey = NamespacedKey(plugin, "chainmail_helmet")
    private var chestplateRecipeKey = NamespacedKey(plugin, "chainmail_chestplate")
    private var leggingsRecipeKey = NamespacedKey(plugin, "chainmail_leggings")
    private var bootsRecipeKey = NamespacedKey(plugin, "chainmail_boots")

    override fun getIdentifier(): String = "CraftableChainmail"
    override fun getCategories(): List<String> = listOf("crafting", "survival")

    override fun onEnable() {
        val helmetRecipe = ShapedRecipe(helmetRecipeKey, ItemStack(Material.CHAINMAIL_HELMET))
        helmetRecipe.shape("CCC", "C C")
        helmetRecipe.setIngredient('C', Material.CHAIN)

        val chestplateRecipe = ShapedRecipe(chestplateRecipeKey, ItemStack(Material.CHAINMAIL_CHESTPLATE))
        chestplateRecipe.shape("C C", "CCC", "CCC")
        chestplateRecipe.setIngredient('C', Material.CHAIN)

        val leggingsRecipe = ShapedRecipe(leggingsRecipeKey, ItemStack(Material.CHAINMAIL_LEGGINGS))
        leggingsRecipe.shape("CCC", "C C", "C C")
        leggingsRecipe.setIngredient('C', Material.CHAIN)

        val bootsRecipe = ShapedRecipe(bootsRecipeKey, ItemStack(Material.CHAINMAIL_BOOTS))
        bootsRecipe.shape("C C", "C C")
        bootsRecipe.setIngredient('C', Material.CHAIN)

        plugin.server.addRecipe(helmetRecipe)
        plugin.server.addRecipe(chestplateRecipe)
        plugin.server.addRecipe(leggingsRecipe)
        plugin.server.addRecipe(bootsRecipe)
    }

    override fun onDisable() {
        plugin.server.removeRecipe(helmetRecipeKey)
        plugin.server.removeRecipe(chestplateRecipeKey)
        plugin.server.removeRecipe(leggingsRecipeKey)
        plugin.server.removeRecipe(bootsRecipeKey)
    }
}